--Blink SQL
--********************************************************************--
--Author:
--CreateTime: 2018-12-04 09:27:19
--Comment: 视频直播解决方案之 视频核心指标监控
--********************************************************************--
CREATE TABLE app_heartbeat_stream_source (
	`ip`                   VARCHAR,
	agent                   VARCHAR,
	roomid                  VARCHAR,
	userid                  VARCHAR,
	abytes                  VARCHAR,
	afcnt                   VARCHAR,
	adrop                   VARCHAR,
	afts                    VARCHAR,
	alat                    VARCHAR,
	vbytes                  VARCHAR,
	vfcnt                   VARCHAR,
	vdrop                   VARCHAR,
	vfts                    VARCHAR,
	vlat                    VARCHAR,
	ublock                  VARCHAR,
	dblock                  VARCHAR,
	`timestamp`             VARCHAR,
	region                  VARCHAR,
    app_ts AS TO_TIMESTAMP(`timestamp`), --定义生成WATERMARK的字段，
    WATERMARK FOR app_ts AS withOffset(app_ts, 10000) --WATERMARK比数据时间线性增加10S
) WITH (
	type = 'datahub',
	endPoint = 'http://dh-cn-hangzhou.aliyun-inc.com',
	roleArn='',
	project = '',
	topic = ''
);


CREATE VIEW view_app_heartbeat_stream AS
SELECT
    ip,
    agent,
    CAST(roomid AS BIGINT)as roomid,
    CAST(userid AS BIGINT)as userid,
    CAST(abytes AS BIGINT)as abytes,
    CAST(afcnt AS BIGINT)as afcnt,
    CAST(adrop AS BIGINT)as adrop,
    unix_timestamp(afts) as afts,
    CAST(alat AS BIGINT)AS alat,
    CAST(vbytes AS BIGINT)as vbytes,
    CAST(vfcnt AS BIGINT)as vfcnt,
    CAST(vdrop AS BIGINT)as vdrop,
    unix_timestamp(vfts)as vfts,
    CAST(vlat AS BIGINT)as vlat,
    CAST(ublock AS BIGINT)as ublock,
    CAST(dblock AS BIGINT)as dblock,
    app_ts,
	region
FROM
    app_heartbeat_stream_source;
------------------------------------------------------------------------------------------------------
--统计房间故障，故障包括卡顿、丢帧、音视频不同步信息，使用10分钟一个窗口进行统计。
CREATE VIEW room_error_statistics_10min AS
SELECT
    CAST(TUMBLE_START(app_ts, INTERVAL '10' MINUTE) as VARCHAR) as app_ts,
    roomid,
    SUM(ublock) as ublock, --统计10分钟内上行卡顿次数
    SUM(dblock) as dblock, --统计10分钟内下行卡顿次数
    SUM(adrop) as adrop, --统计10分钟内音频丢包次数
    SUM(vdrop) as vdrop, --统计10分钟内视频丢包次数
    SUM(alat) as alat, --统计10分钟内音频延迟
    SUM(vlat) as vlat  --统计10分钟内音频延迟
FROM
    view_app_heartbeat_stream
GROUP BY
    TUMBLE(app_ts,INTERVAL '10' MINUTE), roomid;

CREATE TABLE output_1 (
	app_ts             VARCHAR,
	roomid             BIGINT,
	ublock             BIGINT,
	dblock             BIGINT,
	adrop              BIGINT,
	vdrop              BIGINT,
	alat               BIGINT,
	vlat               BIGINT,
	PRIMARY KEY (roomid)
) WITH (
	type= 'rds',
	url = 'jdbc:mysql://*:3306/*',
	userName = '',
	password = '',
	tableName = ''
);

INSERT INTO output_1
SELECT * FROM room_error_statistics_10min;

------------------------------------------------------------------------------------------------------
--分地域统计数据端到端延迟平均情况，每10分钟统计音频、视频平均延迟情况。
CREATE VIEW region_lat_statistics_10min AS
SELECT 
    CAST(TUMBLE_START(app_ts, INTERVAL '10' MINUTE) as VARCHAR) as app_ts,
    region,
    SUM(alat)/COUNT(alat) as alat,
    SUM(vlat)/COUNT(vlat) as vlat
FROM
    view_app_heartbeat_stream
GROUP BY
    TUMBLE(app_ts, INTERVAL '10' MINUTE), region;

CREATE TABLE output_2 (
    app_ts VARCHAR,
    region VARCHAR,
    alat   DOUBLE,
    vlat   DOUBLE
) WITH (
    type ='PRINT'
);

INSERT INTO output_2
SELECT * FROM region_lat_statistics_10min;

------------------------------------------------------------------------------------------------------
--统计实时整体卡顿率，即出现卡顿的在线用户数/在线总用户数*100%，通过此指标可以衡量当前卡顿影响的人群范围。
CREATE VIEW block_total_statistics_10min AS
SELECT
    CAST(TUMBLE_START(app_ts, INTERVAL '10' MINUTE) as VARCHAR) as app_ts,
    SUM(IF(ublock <> 0 OR dblock <> 0, 1, 0)) / CAST(COUNT(DISTINCT userid) AS DOUBLE) as block_rate  
    --COUNT(DISTINCT) 需要在1.4.4版本以上Blink才能支持
FROM
    view_app_heartbeat_stream
GROUP BY
    TUMBLE(app_ts, INTERVAL '10' MINUTE);

CREATE TABLE output_3 (
	app_ts                VARCHAR,
	block_rate            DOUBLE
) WITH (
	type= 'rds',
	url = 'jdbc:mysql://*:3306/*',
	userName = '',
	password = '',
	tableName = ''
);


INSERT INTO output_3
SELECT * FROM block_total_statistics_10min;

-----------------------------------------------------------------------------------------------------
--统计人均卡顿次数，即在线卡顿总次数/在线用户数，通过此指标可以从卡顿频次上衡量整体的卡顿严重程度。
CREATE VIEW block_peruser_statistics_10min AS
SELECT
    CAST(TUMBLE_START(app_ts, INTERVAL '10' MINUTE) as VARCHAR) as app_ts,
    SUM(ublock+dblock) / CAST(COUNT(DISTINCT userid) AS DOUBLE) as block_peruser
    --COUNT(DISTINCT) 需要在1.4.4版本以上Blink才能支持
FROM
    view_app_heartbeat_stream
GROUP BY
    TUMBLE(app_ts, INTERVAL '10' MINUTE);

CREATE TABLE output_4 (
	app_ts                   VARCHAR,
	block_peruser            DOUBLE
) WITH (
	type= 'rds',
	url = 'jdbc:mysql://*:3306/*',
	userName = '',
	password = '',
	tableName = ''
);


INSERT INTO output_4
SELECT * FROM block_peruser_statistics_10min;